import java.awt.*;



/**
 * CurveDrawer does the real work.
 */
public class CurveDrawer extends Panel {
    float e = .25f;		// Eccentricity
    float param = 1.0f;		// Curve parameter (for Cassini ovals)
    boolean cassini;		// Ovals of Cassini (constant product)?
				//  (if not, we're drawing ellipses, or
				//   ovals of Descartes.)
    float a = 100;		// Semimajor axis
    float b = 100;		// Semiminor axis
    float focalratio = 1;
    float targetlen;
    TwoVector origin;		// Point near middle of drawing area
    TwoVector foci[];		// Foci, at origin +/- e*a
    TwoVector pencil;		// Location of pencil
    boolean drawstring;		// Draw the string?
    boolean grabbed;		// Dragging the foci?
    boolean onellipse;		// Dragging on the ellipse?
    boolean show;		// Showing the curve?
    int brighter = 0;		// Brighten the curve? (wide lines,light color)
    Locus drawn;		// List of points user has dragged along ellipse
    OvalLocus curve;		// List of points along curve itself
    Image offscreen;		// Image for double-buffered drawing
    TwoVector tvec = new TwoVector(); // Scratch space
    int plumbx[] = new int[3], plumby[] = new int[3];

		// Drawing parameters
    int dotradius = 2;		// Size (pixels) of dots at foci
    int grabradius = 6;		// Max error (pixels) to grab a focus
    float egraberror = 5;	// Max error (pixels) to grab ellipse
    float espliterror = .05f;	// Max error (pixels) to abandon chords
    float ticksper = 10.01f;	// Tick-marks per unit (a) distance
    boolean doublebuffered = true;
    Color background = Color.black;
    Color foreground = Color.white;
    Color lcolor = Color.yellow;
    Color rcolor = Color.green;
    Color ellipsecolor = Color.darkGray;
    Color brightellipsecolor = Color.lightGray;
    Color drawncolor = Color.red;
    Color dregcolor = Color.white;
    Font tagfont = new Font("Courier", Font.BOLD, 14);


    public CurveDrawer( ) {
	foci = new TwoVector[2];
	foci[0] = new TwoVector(); foci[1] = new TwoVector();
	pencil = new TwoVector();
	this.setBackground( this.background );
	this.setForeground( this.foreground );
	drawn = new Locus();
	curve = new OvalLocus();
	tvec = new TwoVector();
	origin = new TwoVector();
	show = true;
    }

    public void showCurve( boolean show ) {
	this.show = show;
	repaint();
    }

    public void setBright( int bright ) {
	this.brighter = bright;
	repaint();
    }

    public void setE( float e ) {
	drawn.erase();
	this.e = Math.max( 0, Math.min( 1, e ) );
	repaint();
    }

    public void setParam( float param ) {
	if( this.param == param )
	    return;
	this.param = param;
	drawstring = false;
	drawn.erase();
	// Ensure that foci are in a possible position.
	float f[] = foci[0].getData();
	dragfocus( (int)f[0], (int)f[1] );
	repaint();
    }

    public void setCassini( boolean cassini ) {
	this.cassini = cassini;
	drawn.erase();
	repaint();
    }

    public void drawCurve(Graphics g, Rectangle box) {
	// Fit as well as we can into the given drawing area.
	int x0 = box.x + box.width/2;
	int y0 = box.y + box.height/2;
	int i;
	origin.setData( x0, y0 );

	a = Math.round( Math.min(box.width, box.height) / 2.1f );
	TwoVector X = new TwoVector( a, 0 );

	/* Sanity check */
	if(e < 0 || e > 1)
	    e = 1f;

	b = a * (float)Math.sqrt( 1 - e*e ) ;

	focalratio = cassini ? 1 : param;
	targetlen = cassini ? 0 : (1+param)*a;

	foci[0].lincomb( -e*focalratio,X, 1,origin );
	foci[1].lincomb( e,X, 1,origin );
	float f0[] = foci[0].getData();
	float f1[] = foci[1].getData();

	curve.setParams( origin, foci, a, e, param, cassini );

	g.setFont( tagfont );

	/* Draw foci */
	g.setColor( lcolor );
	g.fillOval( (int)f0[0]-dotradius, (int)f0[1]-dotradius,
			2*dotradius, 2*dotradius );
	g.setColor( rcolor );
	g.fillOval( (int)f1[0]-dotradius, (int)f1[1]-dotradius,
			2*dotradius, 2*dotradius );

	if(show) {
	    /* Draw curve. */
	    g.setColor( brighter>0 ? brightellipsecolor : ellipsecolor );

	    curve.draw( g, brighter>1 );
	}

	if(drawstring) {
	  float p[] = pencil.getData();
	  float d0 = pencil.dist( foci[0] ) / a;
	  float d1 = pencil.dist( foci[1] ) / a;
	  float sofar;

	  g.setColor( lcolor );
	  sofar = drawMarkedLine( g, foci[0], pencil, a/ticksper, 0 );
	  g.drawString( "  " + f3format(d0), 10, 1*16 );

	  g.setColor( rcolor );
	  sofar = drawMarkedLine( g, pencil, foci[1], a/ticksper/focalratio,
					sofar/focalratio ) * focalratio;
	  String sd1;
	  float actuallen;
	  if(cassini) {
	    sd1 = "x " + f3format(d1);
	    actuallen = d0 * d1;
	  } else {
	    actuallen = d0 + focalratio*d1;
	    sd1 = "+ " + f3format(d1);
	    if(param < .999f || param > 1.001f)
		sd1 = sd1 + " x " + f3format(param);
	  }
	  g.drawString( sd1, 10, 2*16 );

	  g.setColor( onellipse ? drawncolor : foreground );
	  g.drawString( "= " + f3format(actuallen), 10, 3*16 );

	  if( sofar+2 < targetlen ) {
	    g.setColor( dregcolor );
	    float foc1[] = foci[1].getData();
	    int fx = (int) foc1[0], fy = (int) foc1[1];
	    g.drawLine( fx, fy, fx, fy + (int)(targetlen - sofar) );
	    fy += (int)(targetlen - sofar);
	    plumbx[0] = fx - 5;  plumby[0] = fy;
	    plumbx[1] = fx + 5;  plumby[1] = fy;
	    plumbx[2] = fx;      plumby[2] = fy + 8;
	    g.fillPolygon( plumbx, plumby, 3 );
	  }
	}

	if(drawn.length() > 0) {
	  g.setColor( drawncolor );
	  drawn.draw( g, brighter>1 );
	}
    }

    /*
     * Given a float, return a string in something like %.3f format
     * (though without extra trailing zeros).
     */
    public static String f3format( float f ) {
	String s = Float.toString( Math.round(f*1000f) / 1000f );
	int i = s.lastIndexOf('.');
	if(i < 0 || i+4 >= s.length())
	    return s;
	return s.substring(0, i+4);
    }

    /*
     * Draw a straight piece of string, with tic-marks spaced at distance
     * "dmark", and assuming we've already traversed length "len0" of the
     * string.  Return the new total length.
     */
    public float drawMarkedLine( Graphics g, TwoVector p0, TwoVector p1, float dmark, float len0 ) {
	float v0[] = p0.getData();
	float v1[] = p1.getData();
	g.drawLine( (int)v0[0], (int)v0[1], (int)v1[0], (int)v1[1] );

	float len = p0.dist(p1);
	if(len == 0 || dmark <= 0)
	    return len0 + len;

	// We'll draw the tick marks normal to the given vector,
	// and with length of half their separation.  So make "athwart"'s
	// length be dmark/2.
	TwoVector along = new TwoVector();
	along.lincomb( .5f*dmark/len,  p0,  -.5f*dmark/len, p1 );

	TwoVector athwart = along.perpTo();
	float nsteps = len / dmark;
	float lenmarks = len0 / dmark;
	float step = ((float)Math.ceil(lenmarks) - lenmarks) / nsteps;
	while(step <= 1) {
	    tvec.lincomb( 1-step, p0, step, p1 );
	    tvec.lincomb( 1, tvec, -.5f, athwart );
	    float tic[] = tvec.getData();
	    int tic0x = Math.round(tic[0]), tic0y = Math.round(tic[1]);
	    tvec.add( athwart );
	    tic = tvec.getData();
	    g.drawLine( tic0x, tic0y, Math.round(tic[0]), Math.round(tic[1]) );
	    step += 1f/nsteps;
	}
	return len0 + len;
    }

    /* If we click near a focus, we drag it around.
     * If we click elsewhere, pretend we're pulling a string.
     */
    public boolean mouseDown( Event ev, int x, int y ) {

	if(origin == null)
		return false;	// Not initialized yet (by drawing)

	// Right-mouse click: print current status.
	if(ev.metaDown()) {
	    System.out.println( "Settings: p=" + param + " e=" + e + " cassini=" + (cassini?"on":"off") + " curve=" + (show?"on":"off") );
	    if(ev.controlDown()) {
		System.out.println( "misc: a " + a + 
		    " foci " + foci[0] + " " + foci[1] +
		    " verts[" + curve.length() + "] " +
		    curve.data()[0] + " " + curve.data()[1] + "  " +
		    curve.data()[2] + " " + curve.data()[3] + "  " +
		    curve.data()[4] + " " + curve.data()[5] + " ..." );
	    }
	    if(ev.shiftDown()) {
		System.out.println("Foci: " + foci[0] + " " + foci[1]);
		for(int i = 0; i < curve.length(); i++)
		  System.out.println(curve.data()[i*2] + " " + curve.data()[i*2+1]);
	    }
	}

	// Are we near either focus?  Let them be dragged around.
	pencil.setData(x, y);

	if(pencil.dist(foci[0]) <= grabradius
				|| pencil.dist(foci[1]) <= grabradius ) {
	    grabbed = true;
	    drawstring = false;
	    drawn.erase();
	    dragfocus(x, y);
	} else {
	    drawstring = true;
	    trackellipse(x, y);
	}

	repaint();
	return true;	// We've handled it.
    }

    public boolean mouseUp( Event ev, int x, int y ) {
	grabbed = false;
	drawn.add( -1, -1 );
	repaint();
	return true;
    }

    public boolean mouseDrag( Event ev, int x, int y ) {
	if(grabbed) {
	    dragfocus(x, y);
	    if(ev.metaDown())
		System.out.println("e=" + e);
	} else {
	    trackellipse(x, y);
	}
	return true;
    }

    void dragfocus( int x, int y ) {
	float or[] = origin.getData();
	float dx = (x - or[0]) / a;
	e = Math.min( 1, (dx > 0) ? dx : -dx/focalratio );
	repaint();
    }


    /* Is the point (x, y) close to the ellipse?
     * If so, make it lie pretty accurately on the ellipse, and
     * add point to "drawn" locus.  Set pencil to that point, too.
     */
    void trackellipse( int x, int y ) {
	pencil.setData(x, y);

	float err = curve.nearcurve( pencil, tvec );
	float gradmag = tvec.mag();
	float trueerr = err / gradmag;

	    // Are we close enough to the ellipse?
	    // If so, use findcurve() to get much closer.

	if( trueerr > -egraberror && curve.findcurve( pencil, err, tvec ) ) {
	    int nlocus = drawn.length();
	    if(nlocus > 0) {
		int locus[] = drawn.data();
		// tvec = chord midpoint.  Is that close enough to curve?
		tvec.setData( locus[2*nlocus-2], locus[2*nlocus-1] );
		tvec.lincomb( .5f, tvec, .5f, pencil );
		err = curve.nearcurve( tvec, tvec );
		if( err*err > tvec.dot(tvec) * espliterror )
		    drawn.add( -1, -1 );
	    }
	    drawn.add( pencil );
	    onellipse = true;
	} else {
	    onellipse = false;
	    drawn.add( -1, -1 );
	}
	repaint();
    }


    public boolean keyDown( Event ev, int key ) {
	if(key == 'q')
	    System.exit(0);
	return false;
    }


    public synchronized void update(Graphics screeng) {
	paint(screeng);
    }

    public void paint(Graphics screeng) {
	Rectangle area = bounds();
	Graphics g;

	if(doublebuffered) {
	    /* If we're double-buffering, we need an (off-screen) image to
	     * draw into.  Keep one lying around in "offscreen".
	     * If that hasn't yet been initialized, or if our size has
	     * changed since then, then create a new one.
	     */
	    if(offscreen == null
			    || offscreen.getWidth(null) != area.width
			    || offscreen.getHeight(null) != area.height) {
		offscreen = createImage(area.width, area.height);
		// If we resize, need to discard our cached locus, too.
		drawn.erase();
	    }
	    /* We'll render into the off-screen image's graphics area.
	     * Start by clearing it to the background color.
	     */
	    g = offscreen.getGraphics();
	} else {
	    // Otherwise, we're single-buffered; render directly to the screen.
	    g = screeng;
	}

	g.setColor(getBackground());
	g.fillRect(0, 0, area.width, area.height);
	g.setColor(getForeground());
	dopaint(g);

	/*
	 * If we were double-buffering, finish by copying the image we just
	 * drew into the relevant piece of the screen.
	 */
	if(doublebuffered) {
	    screeng.drawImage(offscreen, 0, 0, this);
	}
    }

    public void dopaint(Graphics g) {

	long before = System.currentTimeMillis();
	drawCurve( g, bounds() );			// Do all the work!
	long delay = System.currentTimeMillis() - before;
    }

}

/* Might use this later... */

class OvalLocus extends Locus {
    float a, e, p;
    TwoVector origin, foci[];
    boolean cassini = false;
    boolean valid = false;
    TwoVector pt, grad;

    public OvalLocus( ) {
	pt = new TwoVector();
	grad = new TwoVector();
	origin = new TwoVector();
    }

    /**
     * If points on the curve have f(x,y) = 0,
     * return f(x,y) for the given point "pt", and the grad of f at pt.
     * Could use this to take a Newton's method step toward the curve.
     */
    public float nearcurve( TwoVector pt, TwoVector grad ) {
	float d0 = pt.dist(foci[0]);
	float d1 = pt.dist(foci[1]);
	float f = cassini ? d0*d1 - p*a*a : d0 + p*d1 - (1+p)*a;
	if(d0 == 0) d0 = 1;
	if(d1 == 0) d1 = 1;
	if(cassini) {
	    grad.lincomb( d1/d0, foci[0], d0/d1, foci[1] );
	    grad.lincomb( d0/d1 + d1/d0, pt,  -1, grad );
	} else {
	    grad.lincomb( 1/d0, foci[0],  p/d1, foci[1] );
	    grad.lincomb( 1/d0 + p/d1, pt, -1, grad );
	}
	return f;
    }

    public boolean findcurve( TwoVector pt, float f, TwoVector grad ) {
	int limit;
	float stepsize = 1;
	for(limit = 15; --limit > 0; ) {
	    float ptwas[] = pt.getData();
	    float xwas = ptwas[0], ywas = ptwas[1];
	    float gradmag2 = grad.dot( grad );
	    if(gradmag2 == 0) gradmag2 = .001f;
	    pt.lincomb( 1,pt, -stepsize * f / gradmag2, grad );
	    float newf = nearcurve( pt, grad );
	    if(Math.abs(newf) > Math.abs(f)) {
		pt.setData( xwas, ywas );
		stepsize *= .6f;
	    } else {
		stepsize = 1;
		f = newf;
		if(f*f < .01f*gradmag2)
		    return true;
	    }
	}
	return false;
    }

    public int findroots( float roots[] ) {
	float p0[] = foci[0].getData();
	float p1[] = foci[1].getData();
	int i = 0;
	if(cassini) {
	    float t = p*a*a;
	    float mid = (p0[0] + p1[0])/2;
	    float semi = (p1[0] - p0[0])/2;
	    float s2 = semi*semi;
	    roots[i++] = mid - (float) Math.sqrt( s2 + t );
	    if(s2 >= t) {
		float disc = (float) Math.sqrt( s2 - t );
		roots[i++] = mid - disc;
		roots[i++] = mid + disc;
	    }
	    roots[i++] = mid + (float) Math.sqrt( s2 + t );
	} else {
	    float t = (1+p)*a;
	    float s = p1[0] - p0[0];
	    if(t >= p*s) {
		// Root to left of foci[0].
		roots[i++] = p0[0] - (t - p*s)/(1+p);
	    }
	    if((s < t && t < p*s) || (p*s < t && t < s)) {
		// Root between the foci.
		roots[i++] = p0[0] + (t - p*s) / (1 - p);
	    }
	    if(t >= s) {
		// Root to the right of foci[1].
		roots[i++] = p1[0] + (t - s)/(1+p);
	    }
	}
	return i;
    }


    public void setParams( TwoVector origin, TwoVector foci[],
				float a, float e, float p, boolean cassini ) {
	if(a==this.a && e==this.e && p==this.p && cassini==this.cassini
		&& origin.dist( this.origin ) == 0)
	    return;

	valid = false;
	this.erase();
	this.a = a;
	this.e = e;
	this.p = p;
	this.cassini = cassini;
	this.origin.copyFrom( origin );
	this.foci = foci;
    }

    public void draw( Graphics g, boolean wider ) {
	if(!valid)
	    recalculate();
	super.draw( g, wider );
    }

    void recalculate() {
	int px, py;
	float fx, fy;
	float xroots[] = new float[4];
	int r, nroots;
	float stepsize = 45;
	TwoVector prevpt = new TwoVector();
	TwoVector along = new TwoVector();
	TwoVector midpt = new TwoVector();
	TwoVector midgrad = new TwoVector();
	float y0 = foci[0].getData()[1];

	this.erase();

	nroots = findroots( xroots );

	for(r = 0; r < nroots-1; r += 2) {
	    int lbase = this.nlocus;
	    float prevy = y0;
	    float mid = (xroots[r+1] + xroots[r]) / 2;
	    float span = (xroots[r+1] - xroots[r]) / 2;
	    boolean done = false;
	    int tries = 1000;

	    this.add( xroots[r], y0 );

	    prevpt.setData( xroots[r], y0 );
	    along.setData( .5f, .866f );	// Mostly-upward vector
	    for(tries = 1000; --tries >= 0; ) {
		float mag = along.mag();
		float step = stepsize / (mag == 0 ? 10 : mag);
		int lim = 15;
		float midf;

		do {
		    pt.lincomb( 1, prevpt, step, along );
		    float f = nearcurve( pt, grad );
		    if( findcurve( pt, f, grad ) ) {
			midpt.lincomb( .5f, pt, .5f, prevpt );
			midf = nearcurve( midpt, midgrad );
			if( midf*midf < .01f * midgrad.dot( midgrad ) ) {
			    break;
			}
		    }
		    // Oops.  Take a smaller step.
		    step *= .6f;
		} while(--lim > 0);
		if(lim <= 0) {
System.out.println("Lost the curve at " + a + " " + e + " " + p + " " + pt);
		    // Hack the locus[] pointer: leave just the
		    // initial root in here, so we draw a simple line
		    // from the leftmost root to the rightmost root.
		    // This is a good way to deal with degenerate ellipses.
		    nlocus = lbase + 2;
		    break;
		}

		// Have we crossed back through the line of symmetry yet?
		if(pt.getData()[1] <= y0)
		    break;

		this.add( pt );
		prevpt.copyFrom( pt );
		along.setPerpTo( grad );
	    }
	    this.add( xroots[r+1], y0 );
	    // Produce Y-mirrored copy of all the points we've created so far.
	    int nhalf = nlocus;
	    int k;
	    for(k = nhalf; (k -= 2) >= lbase; ) {
	      if(locus[k] == -1) {
		    add( -1, -1 );
	      } else {
		    add( locus[k], 2*y0 - locus[k+1] );
	      }
	    }
	    add( -1, -1 );
	}
	valid = true;
    }
}

class Locus {

    int [] locus;
    int nlocus;

    /**
     * Add a point to our list of line segments along the locus.
     * Each new point extends the polygonal curve.  Adding the special point
     * (-1, -1) introduces a gap in the curve.  add() ignores adjacent
     * duplicate points.
     */

    public Locus() {
	locus = new int[1000];
	nlocus = 0;
    }

    public int length() {
	return nlocus/2;
    }

    public int[] data() {
	return locus;
    }

    public void erase() {
	nlocus = 0;
    }

    public void draw( Graphics g, boolean wider ) {
	int i;
        for(i = 2; i < nlocus; i += 2) {
            /* Negative X values indicate that the locus isn't continuous,
             * so don't draw there.
             */
	    if(Math.abs(locus[i-2]+locus[i-1]+locus[i]+locus[i+1]) > 6000) {
	      System.out.println("trouble: " + locus[i-2] + " " + locus[i-1] + "  " + locus[i] + " " + locus[i+1]);
	      break;
	    }
            else if(locus[i-2] > 0 && locus[i] > 0)
              g.drawLine( locus[i-2],locus[i-1], locus[i],locus[i+1] );
	}
	if(wider) {
	  int dx, dy, x0,y0,x1,y1, lastx=0,lasty=0;
	  boolean joint = false;
	  for(i = 2; i < nlocus; i += 2) {
	    if((x0 = locus[i-2]) > 0 && (x1 = locus[i]) > 0) {
		y0 = locus[i-1];
		y1 = locus[i+1];
		dy = x1-x0;
		dx = x0-x1;
		if(dx < 0) {
		    if(dy < dx) { dy = -1; dx = 0; }
		    else if(dy > -dx) { dy = 1; dx = 0; }
		    else { dy = 0; dx = -1; }
		} else {
		    if(dy > dx) { dy = 1; dx = 0; }
		    else if(dy < -dx) { dy = -1; dx = 0; }
		    else { dy = 0; dx = 1; }
		}
		if(!joint) {
		    lastx = x0 + dx;  lasty = y0 + dy;
		}
		g.drawLine(lastx,lasty, x1+dx,y1+dy);
		lastx = x1+dx; lasty = y1+dy;
		joint = true;
	    } else {
		joint = false;
	    }
	  }
	}
    }

    public void add( float x, float y ) {
	add( Math.round(x), Math.round(y) );
    }

    public void add( TwoVector pt ) {
	float xy[] = pt.getData();
	add( Math.round(xy[0]), Math.round(xy[1]) );
    }

    public void add( int x, int y ) {
	if(nlocus>0 && locus[nlocus-2] == x && locus[nlocus-1] == y)
	    return;

	if (nlocus >= locus.length) {
	    int nl[] = new int[ locus.length * 2 ];
	    System.arraycopy( locus, 0, nl, 0, nlocus );
	    locus = nl;
	}

	locus[nlocus++] = x;
	locus[nlocus++] = y;
    }
}

/**
 * N-dimensional vector class, with basic linear operations.
 */
class NVector extends Object implements Cloneable {
   // Our sole instance variable.
   float data[];

   /**
    * This isn't very useful, but Java requires that we offer a
    * constructor with no arguments if we have subclasses.
    * See the Nutshell book, p. 65.
    */

   public NVector() {
    /* We could explicitly invoke one of the other constructors here.
     * For example, writing
     *	this(3);
     * would mean that a ``new NVector()'' would be 3-D by default.
     */
   }

   public NVector(int dim) {
	data = new float[dim];
   }

   public NVector(float initialdata[]) {
	copyData( initialdata );
   }

   /**
    * Return a copy of this NVector.
    */
   public Object clone() {
	return new NVector(this.data);
   }

   /**
    * Get vector data.
    */
   float [] getData() {
	return this.data;
   }

   /**
    * Set vector data.
    */
   void setData(float newdata[]) {
	data = newdata;
   }

   /**
    * Set vector data, making a copy.
    */
   void copyData(float newdata[]) {
	data = new float[newdata.length];
	for(int i = 0; i < data.length; i++)
	    data[i] = newdata[i];
   }

   /**
    * Set vector's data to match that of some other vector.
    */
   public NVector copyFrom( NVector v ) {
	copyData( v.data );
	return this;
   }

   /**
    * Return dimension of this vector.
    */
   public int dim() {
	return this.data.length;	// Ask our array for its length
   }

   /**
    * (Euclidean) inner product of this vector with another.
    */
   float dot(NVector v) {
	float sum = 0;
	int i;
	int dim = v.dim();
	if(dim > this.dim())	//  "this.dim()" is synonymous with "dim()"
	    dim = this.dim();
	for(i = dim; --i >= 0; )
	    sum += data[i] * v.data[i];
	return sum;
   }

   /**
    * (Euclidean) magnitude of this vector.
    */
   public float mag() {
	return (float)Math.sqrt(dot(this)); // Take inner product with self.
   }

   /**
    * Set vector to be a unit vector (unless it's the zero vector)
    * Return original magnitude.
    */
   public float unitize() {
	float mag = mag();
	if(mag > 0) {
	    int i;
	    for(i = this.dim(); --i >= 0; )
		data[i] /= mag;
	}
	return mag;
   }

   /**
    * Linear combination: this = a*av + b*bv
    */
   public void lincomb(float a, NVector av, float b, NVector bv) {
	int dim = Math.max(av.dim(), bv.dim());
	float newdata[];
	newdata = (data != null && data.length == dim) ? data : new float[dim];
	for(int i = 0; i < dim; i++) {
	    newdata[i] = a*av.data[i] + b*bv.data[i];
	}
	data = newdata;
   }

   /**
    * Linear combination: this = a*this + b*bv
    */
   public void lincomb(float a, float b, NVector bv) {
	lincomb(a, this, b, bv);
   }

   /**
    * Subtract another vector v from this one.
    */
   public void subtract(NVector v) {
	lincomb(1,this, -1,v);
   }

   /**
    * Add vector v to this one.
    */
   public void add(NVector v) {
	lincomb(1,this, 1,v);
   }
}

class TwoVector extends NVector {


   public TwoVector() {
	data = new float[2];
   }

   public TwoVector( TwoVector clonefrom ) {
	this();
	data[0] = clonefrom.data[0];
	data[1] = clonefrom.data[1];
   }

   public TwoVector(float x, float y) {
	this();
	data[0] = x;
	data[1] = y;
   }

   public Object clone() {
	return new TwoVector(this.data);
   }

   // Let the NVector( float[] data ) code do the work.
   // We could check that the data supplied here really has two components.
   public TwoVector( float[] data ) {
	super( data );
   }

   /**
    * Return new vector perpendicular to this one
    */
   TwoVector perpTo() {
	return new TwoVector(data[1], -data[0]);
   }

   void setPerpTo( TwoVector tv ) {
	float t = tv.data[0];
	data[0] = tv.data[1];
	data[1] = -t;
   }

   void setData(float x, float y) {
	data[0] = x;
	data[1] = y;
   }

   public float dist( TwoVector another ) {
	float dx = data[0] - another.data[0];
	float dy = data[1] - another.data[1];
	return (float) Math.sqrt( dx*dx + dy*dy );
   }

   public TwoVector copyFrom( TwoVector v ) {
	copyData( v.data );
	return this;
   }

   public String toString() {
	return "[" + data[0] + "," + data[1] + "]";
   }
}
