import java.awt.*;
import java.util.StringTokenizer;
import java.util.Hashtable;


interface ParamSetting {
    public void setParam( float value );
}


/**
 * FlakePanel encapsulates the graphical stuff of interest, and any
 * user-interface we might add.  We could create multiple instances of it.
 */
public class CurvePanel extends Panel implements ParamSetting {
    CurveDrawer cd;
    Panel bottom;
    Panel left;
    Checkbox showCurve, brighten;
    Choice curvetype;
    int prefitem;
    CurveScrollbar param;
    Hashtable menusets;

    CurvePanel( ) {
	setLayout( new BorderLayout() );
	cd = new CurveDrawer( );
	add("Center", cd);

	bottom = new Panel();
	add("South", bottom);
	bottom.setLayout( new BorderLayout( ) );

	left = new Panel();
	bottom.add( "West", left );
	left.setLayout( new FlowLayout( FlowLayout.LEFT ) );

	brighten = new Checkbox("Brighter");
	left.add( brighten );

	showCurve = new Checkbox("Show curve");
	left.add( showCurve );

	clearChoices();

	param = new CurveScrollbar( this, Scrollbar.HORIZONTAL, 1.0f, .1f, .25f, 2.5f );
	bottom.add( "Center", param );

	addChoice( "Ellipse", "slider=false p=1 cassini=false" );
	addChoice( "Descartes Oval", "slider=true cassini=false" );
	addChoice( "Cassini Oval", "slider=true cassini=true" );

	prefitem = -1;

	curvetype.select( 0 );	// Select ellipse initially.
	applySettings( "p=1 e=.5 slider=off curve=on cassini=off" );
    }

    public void clearChoices() {
	if(curvetype != null)
		left.remove( curvetype );
	curvetype = new Choice();
	left.add( curvetype );
	prefitem = -1;
	this.menusets = new Hashtable();
    }
	
    static float getfloat( String str, float dflt ) {
	float v = dflt;
	try {
	    int i = str.indexOf("=") + 1;
	    v = Float.valueOf( str.substring( i ) ).floatValue();
	} catch(Exception exc) {
	}
	return v;
    }

    static int getint( String str, int dflt ) {
	int v = dflt;
	int i = str.indexOf("=") + 1;
	String s = null;
	try {
	    s = str.substring( i );
	    if(s.equals("on") || s.equals("true")) v = 1;
	    else if(s.equals("off") || s.equals("false")) v = 0;
	    else v = Integer.parseInt( s );
	} catch(Exception exc) {
	}
	return v;
    }

    static boolean getboolean( String str, boolean dflt ) {
	boolean v = dflt;
	int i = str.indexOf("=") + 1;
	String s = null;
	try {
	    s = str.substring( i );
	    if(s.equals("on") || s.equals("true")) v = true;
	    else if(s.equals("off") || s.equals("false")) v = false;
	    else v = (Integer.parseInt( s ) != 0);
	} catch(Exception exc) {
	}
	return v;
    }

    public void applySettings( String settings ) {
	boolean setE = false;
	float e = 0;

	StringTokenizer st = new StringTokenizer( settings, ",; \t\r\n" );

	while(st.hasMoreTokens()) {
	    String tok = st.nextToken().toLowerCase().trim();
	    if(tok.startsWith("e=")) {
		e = getfloat( tok, .5f );
		setE = true;
	    }
	    else if(tok.startsWith("p=")) {
		param.setValue( getfloat( tok, 1.0f ) );
		cd.setParam( param.getValue() );
	    }
	    else if(tok.startsWith("curve=")) {
		boolean on = getboolean( tok, true );
		showCurve.setState( on );
		cd.showCurve( on );
	    }
	    else if(tok.startsWith("slider=")) {
		if(getboolean( tok, true ))
		    param.show();
		else param.hide();
		validate();
		repaint();
	    }
	    else if(tok.startsWith("cassini=") || tok.startsWith("c=")) {
		cd.setCassini( getboolean( tok, false ) );
	    }
	    else if(tok.startsWith("bright=")) {
		int on = getint( tok, false );
		cd.setBright( on );
		brighten.setState( on );
	    }
	    else {
		System.out.println("didn't recognize '" + tok +
		"'; expected e=V or p=V or curve=B or slider=B or cassini=B or bright=B, " +
		"where V is a float, B a boolean (true/false or on/off or 1/0)");
	    }
	}
	// Must set eccentricity last -- CurveDrawer may mess with it
	// if we change Cassini mode or param values afterward.
	if(setE)
	    cd.setE( e );
    }

    public void addChoice( String label, String settings ) {
	menusets.put( label, settings );
	curvetype.addItem( label );
	if(prefitem < 0) {
	    prefitem = curvetype.countItems() - 1;
	    curvetype.select( label );
	    applySettings( settings );
	}
    }

    public void setParam( float param ) {
	cd.setParam( param );
    }

    public boolean action( Event ev, Object arg ) {
	if(ev.target == showCurve) {
	    cd.showCurve( showCurve.getState() );
	    return true;
	} else if(ev.target == brighten) {
	    cd.setBright( brighten.getState() );
	    return true;
	} else if(ev.target == curvetype) {
	    String s = (String) menusets.get( curvetype.getSelectedItem() );
	    if( s != null ) {
		applySettings( s );
	    }
	    return true;
	}
	return super.action( ev, arg );
    }

}

class CurveScrollbar extends Panel {
    ParamSetting ps;
    Scrollbar sb;
    TextField tf;
    String oldtext;
    float val, min, max;

    public CurveScrollbar( ParamSetting ps, int orient, float value, float vis, float min, float max ) {
	this.ps = ps;
	sb = new Scrollbar( orient, (int)(100*value), (int)(100*vis), (int)(100*min), (int)(100*max) );
	setLayout( new BorderLayout( ) );
	add( "Center", sb );
	tf = new TextField( 4 );
	add( "West", tf );
	this.min = min;
	this.max = max;
	this.val = value+1;
	setValue( value );
    }

    public void setValue( float value ) {
	if(value < min) value = min;
	if(value > max) value = max;
	if(value != this.val) {
	    this.val = value;
	    sb.setValue( (int)(100*value) );
	    tf.setText( Float.toString( value ) );
	    oldtext = tf.getText();
	    if(ps != null) ps.setParam( value );
	}
    }

    public float getValue( ) {
	return this.val;
    }

    public boolean handleEvent( Event ev ) {
	if(ev.target == sb) {
	    setValue( sb.getValue() / 100.0f );
	    return true;
	}
	if(! tf.getText().equals( oldtext ) &&
		((ev.id == Event.KEY_PRESS && ev.key == Event.ENTER) ||
		 ev.id == Event.LOST_FOCUS || ev.id == Event.MOUSE_EXIT)) {
	    try {
		float v = Float.valueOf( tf.getText().trim() ).floatValue();
		setValue( v );
	    } catch( NumberFormatException exc ) {
		tf.setText( oldtext );
	    }
	    return true;
	}
	return super.handleEvent( ev );
    }
}
